\name{TraceSurface}
\alias{TraceSurface}
\title{Function to trace the skin in a MRT image}
\description{Returns the surface points and a triangulation of the skin surface.}
\usage{
TraceSurface(nifti_img, start_slice, end_slice, poly_n=10, thres = 0.3, samples_slice=30)
}
\arguments{
\item{nifti_img}{the MRT image (supported Rniftilib)}
\item{start_slice}{slice to start contour tracing}
\item{end_slice}{slice to end contour tracing}
\item{poly_n}{number of points to specify initial outline}
\item{thres}{threshold to detect grey level (in percent of the difference between min and max values)}
\item{samples_slice}{equally distributed points per slice on the skin}
}
\details{

}
\value{
 A list with entries:
  \item{vertices}{point coordinates (nx3 matrix)}
  \item{traingles}{indices of the vertices belonging to a triangle (nx3 matrix)}
}
\references{no references }
\author{Oliver Granert <o.granert <at> neurologie.uni-kiel.de> }
\note{ no notes }
\seealso{
  \code{\link{TraceSurfaceGUI}},
  \code{\link{DisplaySurface}},
}
\examples{
\dontrun{
library(Rniftilib)
library(Rniftitools)

Image <- nifti.image.read(file.choose())

# define start and end slices (section to vectorize)
StartSlice <- 15;
EndSlice <- dim(Image)[3]-15;

# vectorize skin...
Surface <- TraceSurface(Image, StartSlice, EndSlice, 10, 0.3);

DisplaySurface(Surface);

# save surface data in R data format
save(Surface, file=file.choose());
}
}
